;****************************************************************************
; Progetto:	Convertitore analogico/Digitale 
; Autore:	Jumbo elettronica
; Data:		26-3-99
;
; release:	1.0
;***************************************************************************
  list F=INHX8M,P=16C84
;
;----------------------------------------------------------------
RTCC   EQU 01H                 ; Real Time Clock Counter
PCL    EQU 02H                 ; Program Counter
STAT   EQU 03H                 ; Registro di stato
PORTA  EQU 05H                 ; Porta A
PORTB  EQU 06H                 ; Porta B
EDATA  EQU 08H                 ; Data EEPROM
EADR   EQU 09H                 ; Address EEPROM
ECON1  EQU 88H                 ; Stato delle operaz. in EEPROM
ECON2  EQU 89H                 ; Vedi ECON1
INTCON EQU 0BH                 ; Registro abilitazione interrupt
TR_A   EQU 85H                 ; Tris A
TR_B   EQU 86H                 ; Tris B
OPTIO  EQU 81H                 ; Registro OPTION

;----------------------------------------------------------------
w	equ	.00
f	equ	.01

;----------------------------------------------------------------
FR01	EQU 	0CH             ; |
FR02   	EQU 	0DH             ; | buffer per display 
BUFF   	EQU 	0EH             ; |
		
unita	equ	0fh		; 
decine	equ	10h		; 
centinaia	equ	11h	; 
;	equ	12h		;

;FR10	EQU	13H		; FLAG 
;FR11	EQU	14H		; FLAG
;FR03	EQU 	15H             ;
;	equ	16h		; 
volt	equ	17h		;
;	equ	18h		;
;	equ	19h		;

buff1		equ	1ah	;
buff2		equ	1bh	;
buff3		equ	1ch	;
buff4		equ	1dh
;		equ	1eh	; 
RxReg		equ	1fh	; 
RxReg1       	equ     20h 	; Data received
RxReg2       	equ     21h 	; Data received
Count		equ     22h	; number Tx/Rx counter
DlyCnt      	equ	23h     ; general delay counter

;----------------------------------------------------------------
#define RP0     STAT,5          ; Flag selezione banco ram
#define	zero	STAT,2
#define	carry	STAT,0	
#define CLK     PORTA,3         ; Clock per display
#define RS	PORTA,2         ; Register Select per display
#define C1      PORTB,4       ; Colonna 1
#define C2      PORTB,5       ; Colonna 2
#define C3      PORTB,6       ; Colonna 3
#define wr      PORTA,2       ; 
#define cs      PORTA,3       ; 
#define input	PORTA,0       ; 
	
;----------------------------------------------------------------
       ORG       0             ;
       goto      START         ;Reset vector
                               ;
       ORG       4             ;Interrupt vector
       retfie   
               ;
;***************************************************************************


;--------- subroutine RITARDO 20 mS ----------------
; 
;---------------------------------------------------
DELA2M movlw     .20           ; 
       movwf     FR01          ; 
       movlw     .180          ; 
DEL0   movwf     FR02          ; 
DEL1   clrwdt                  ; 
       nop                     ; 
       decfsz    FR02,f        ; 
       goto      DEL1          ; 
       decfsz    FR01,f        ; 
       goto      DEL0          ; 
       return                  ; 

;--------- subroutine RITARDO 100 mS ----------------
; 
;---------------------------------------------------
DELA3M movlw     .65           ; 
       movwf     FR01          ; 
       movlw     0ffh          ; 
DEL2   movwf     FR02          ; 
DEL3   clrwdt                  ; 
       nop                     ; 
       decfsz    FR02,f        ; 
       goto      DEL3          ; 
       decfsz    FR01,f        ; 
       goto      DEL2          ; 
       return                  ; 
	
	
;--- simulazione clock fase 2 in LCD e ritardo di 10 mS --------
CLOCK2
	bcf      CLK            ;
	swapf	BUFF,1		;inverte nible
        movf     BUFF,W         ; carica dato da scivere su W
	andlw	b'00001111'	;scarica 1 nible	
        movwf    PORTB          ;
        nop                     ;
        bsf      CLK            ; clock fase 2
        nop                     ;
        bcf      CLK            ;
	swapf	BUFF,1
        movf     BUFF,W         ;
	andlw	b'00001111'	;scarica 2 nible
        movwf    PORTB          ;
        nop                     ;
        bsf      CLK            ; clock fase 2
        nop                     ;
        bcf      CLK            ;
        movlw    .40            ; carica 50 in W
        movwf    FR01           ; copia W in FR01
DELC0   movwf    FR02           ; copia W in FR02
DELC1   nop                     ; nessuna operazione
        clrwdt                  ; azzera watchdog
        nop                     ; nessuna operazione
        decfsz   FR02,f         ; dec. FR02, salta se 0
        goto     DELC1          ; vai a DELC1
        decfsz   FR01,f         ; dec. FR01, salta se 0
        goto     DELC0          ; cai a DELC0
        return                  ; ritorna da subroutine
;--------- Routine di reset per il display ------------------
RESLCD 
       bcf      RS             ; Istruzione
       movlw    028H           ; 4 bit data length
       movwf    BUFF           ; in BUFFER
       call     CLOCK2         ; chiama sub. CLOCK2
       movlw    01H            ; Reset
       movwf    BUFF           ; in BUFFER
       call     CLOCK2         ; chiama sub. CLOCK2
       movlw    0EH            ; Display on, cursor on, blink off
       movwf    BUFF           ; in BUFFER
       call     CLOCK2         ; chiama CLOCK2
       return                  ; ritorna da subroutine

;------ Routine per la scrittura di un carattere --------

WRITE
	movwf    BUFF           ; copia W in BUFFER
       bsf      RS             ; Data
       call     CLOCK2         ; E
       bcf      RS             ; RS = 0
       movlw    06H            ; carica W con 6
       movwf    BUFF           ; copia W in BUFFER
       call     CLOCK2         ; chiama sub. CLOCK
       return                  ; ritorna da subroutine


;-------------------------------
lpatt
	bcf      RS             ; Istruzione
       	movlw    0fH            ; Display ON, cursor OFF, blink ON
       	movwf    BUFF           ; in BUFFER
       	call     CLOCK2         ; chiama CLOCK2
       	return                  ; ritorna da subroutine




;**********************************************************************
;                    START PROGRAM
;**********************************************************************

START
	bsf       STAT,5        ; Seleziona SRAM banco 1
	movlw     b'00001'      ; RA out  
	movwf     TR_A          ; 

	movlw     b'11110000'   ; RB 0-3=out ; 4-7=in
	movwf     TR_B          ;

	clrf      INTCON        ; Disabilita interrupt
	movlw     b'01000110'   ; Prescaler 1:128
	movwf     OPTIO         ; Copia W in OPTION
	bcf       STAT,5        ; Seleziona SRAM banco 0
	movlw     .128          ; Settaggio iniziale RTCC
	movwf     RTCC          ;
		
;**********************************************************************
;                      main program  
;**********************************************************************
MAIN
	call      DELA2M        ; Attesa 40ms
	call      DELA2M   
	call      RESLCD        ; Reset LCD
	movlw     'C'           ;  
	call      WRITE         ; 
	movlw     'o'           ;  
	call      WRITE         ;  
	movlw     'd'           ;  
	call      WRITE         ;  
	movlw     'e'           ; 
	call      WRITE         ;  
	movlw	  ':'
	call	  WRITE
LPATT
	clrwdt			;
	clrf	PORTB
	bcf	cs
	bcf	wr
	call	DELA2M
	bsf	wr
	call	adc1 
	movf	buff2,w
	call	_dividi
	call	scrivi2
	call	_volt
	movf	buff2,w
	call	_dividi
	call	scrivi4
	goto	LPATT
	

;****************************   A D C   *******************************
adc1
	clrf	buff1
	clrf	buff2
adc2
	call	DELA3M
	call	DELA3M

	rrf	buff2,f
;	call	scrivi3
	movf	buff1,w
	movwf	PORTB
	call	DELA2M
	btfss	input
	goto	adc3
	bsf	buff2,7 	
	
	goto	adc4
adc3
	bcf	buff2,7
	
adc4
;	call	binario	
	incf	buff1,f
	movlw	.8		;
	xorwf	buff1,w
	btfsc	zero		
	return
	goto	adc2

;************* DIVIDI IN UNITA', DECINE , CENTINAIA (per LCD) ********
_dividi
	movwf	RxReg
	clrf	unita
	clrf	decine
	clrf	centinaia
	incf	RxReg,f
_unit
	decfsz	RxReg,f		;controlla se mem1  a zero!!!
	goto	_unit1
_ret
	return
_unit1
	incf	unita,f		;incrementa unit
	movlw	.10		;e controlla se unit vale 10
	xorwf	unita,w
	btfsc	zero		
	goto	_decine
	goto	_unit
_decine
	incf	decine,f	;incrementa decine 
	clrf	unita		;unit a zero
	movlw	.10		;e controlla se decine vale 10
	xorwf	decine,w
	btfsc	zero
	goto	_centinaia
	goto	_unit
_centinaia
	incf	centinaia,f	;incrementa centinaia 
	clrf	decine		;decine a zero
	goto	_unit		;
	return

;**************************** VISUALIZZA  ************************
scrivi2
				
	bcf	RS		;
	movlw	085H		; 	
	movwf	BUFF		;
	call	CLOCK2		;	
	bsf	RS		;

	movf	centinaia,w	;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE		

	movf	decine,w	;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE	

	movf	unita,w		;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE	
	
	return
;---------------------------		
scrivi3
				
	bcf	RS		;
	movlw	0c0H		; 	
	movwf	BUFF		;
	call	CLOCK2		;	
	bsf	RS		;

	movf	buff1,w		;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE		

	return
;--------------------------- USATO PER VISUALIZZARE LA TENSIONE -----------
scrivi4

	movf	centinaia,w	;correggi volt se centinaia vale 1
	addwf	volt,f		
		
	bcf	RS		;
	movlw	0c1H		; 
	addwf	buff3,w	
	movwf	BUFF		;
	call	CLOCK2		;	
	bsf	RS		;

	movlw     'V'           ;  
	call      WRITE         ; 

	movf	volt,w		;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE		

	movlw     ','           ;  
	call      WRITE         ; 

	movf	decine,w	;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE	

	movf	unita,w		;scrivi il carattere ASCII contenuto in mem.
	addlw	30H
	call	WRITE		
	
	return
;***************** CONVERTE IL VALORE DELL'ADC IN TENSIONE ******************
_volt
	clrf	volt		; 
_volt_1
	movf	buff2,w
	sublw	.50
	btfss	carry		; controlla se il valore vale meno di un volt
	goto	_volt_2

	movf	buff2,w
	addwf	buff2,f
	return
_volt_2
	incf	volt,f
	movlw	.50
	subwf	buff2,f
	goto	_volt_1

;-------------
	return
;
;----------------------------------------------------------------
       END




